package net.zenrindatacom.locationservice.sample.activity;

import android.app.AlertDialog;
import android.os.Bundle;
import android.util.Log;
import android.widget.Toast;

import androidx.preference.DropDownPreference;
import androidx.preference.Preference;

import net.zenrindatacom.locationservice.sample.R;
import net.zenrindatacom.locationservice.sample.preference.LocationLibPreference;
import net.zenrindatacom.locationservice.sample.application.SampleApplication;

import java.io.File;
import java.util.Objects;

/**
 * 位置測位ライブラリ設定画面
 *
 * @author N.Miyahara
 */
public class LocationLibSettingActivity extends EdgeToEdgeCompatActivity {
    /**
     * onCreate
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getSupportFragmentManager().beginTransaction().replace(android.R.id.content, new LocationLibSettingPreferenceFragment()).commit();
    }

    /**
     * 本画面のPreferenceFragmentクラス
     */
    public static class LocationLibSettingPreferenceFragment extends SettingPreferenceFragment implements Preference.OnPreferenceClickListener {
        /**
         * 親クラスのonCreatePreferences()イベント発生時に呼び出される
         * 　本タイミングで初期化に必要な処理を行う。
         */
        @Override
        public void onCreateSettingPreference(String rootKey) {
            //対象とするプリファレンスファイル名をデフォルトから変更
            getPreferenceManager().setSharedPreferencesName(LocationLibPreference.getInstance().getPreferenceName());
            //本設定画面と対になるPreferenceScreen設定ファイル
            setPreferencesFromResource(R.xml.location_lib_setting, rootKey);
            //タップ時のリスナー登録（「ログファイル削除」ボタン）
            Preference preference = findPreference(getString(R.string.prf_lib_key_deleteLog));
            if(preference != null) {
                preference.setOnPreferenceClickListener(this);
            }
            //タップ時のリスナー登録（「デフォルト設定に戻す」ボタン）
            preference = findPreference(getString(R.string.prf_lib_key_returnToDefaultSetting));
            if(preference != null) {
                preference.setOnPreferenceClickListener(this);
            }
        }

        /**
         * PreferenceClickリスナー
         * 　PreferenceScreen上のボタンがタップされるとCALLされる
         *
         * @return イベントハンドリング結果
         */
        @Override
        public boolean onPreferenceClick(Preference preference) {

            Log.i("debug-TAG", "Tap!! (" + preference.getKey() + ")");

            if(preference.getKey().equals(getString(R.string.prf_lib_key_deleteLog))) {
                //「ログファイル削除」ボタンをタップ
                showDeleteLogfileDialog();
                return true;
            } else if(preference.getKey().equals(getString(R.string.prf_lib_key_returnToDefaultSetting))) {
                //「デフォルト設定に戻す」ボタンをタップ
                LocationLibPreference locationLibPreference = LocationLibPreference.getInstance();
                locationLibPreference.returnToDefault();
                // ドロップダウンメニューの表示の初期化
                DropDownPreference dropDownPreference = findPreference(getString(R.string.prf_lib_key_auth_env));
                if (dropDownPreference != null) {
                    String value = locationLibPreference.getString(getString(R.string.prf_lib_key_auth_env), "0");
                    dropDownPreference.setValueIndex(Integer.parseInt(value));
                }
                return true;
            }

            //処理なし
            return false;
        }

        /**
         * ログファイル削除確認ダイアログを表示
         */
        private void showDeleteLogfileDialog() {
            AlertDialog dialog = new AlertDialog.Builder(this.getContext())
                    .setCancelable(false)
                    .setMessage(getString(R.string.message_confirm_delete_log_file))
                    .setPositiveButton(android.R.string.ok, (dialog12, which) -> {
                        //ログファイル削除
                        int deleteItemCount = deleteLogfile();
                        //削除結果を表示
                        if (deleteItemCount > 0) {
                            Toast.makeText(this.getContext(), deleteItemCount + getString(R.string.toast_n_file_deleted), Toast.LENGTH_LONG).show();
                        } else {
                            Toast.makeText(this.getContext(), getString(R.string.toast_delete_file_not_found), Toast.LENGTH_LONG).show();
                        }
                    })
                    .setNegativeButton(android.R.string.cancel, (dialog1, which) -> {
                    })
                    .create();

            dialog.show();
        }

        /**
         * ログファイルの削除
         * 　ライブラリが出力したログファイルを削除する
         *
         * @return 削除したログファイルの数
         */
        private static int deleteLogfile() {

            String deleteFileDirectoryPath = Objects.requireNonNull(SampleApplication.getAppContext().getExternalFilesDir(null)).toString();

            File deleteFileDirectory = new File(deleteFileDirectoryPath);
            final File[] listFiles = deleteFileDirectory.listFiles();
            if (listFiles == null || listFiles.length == 0) {
                return 0;
            }

            int deleteItemCount = 0;
            for (File file : listFiles) {
                String fileName = file.getName();
                String deleteFilePath = deleteFileDirectoryPath + "/" + fileName;
                File deleteFile = new File(deleteFilePath);
                if (deleteFile.exists() && deleteFile.delete()) {
                    deleteItemCount++;
                }
            }

            return deleteItemCount;
        }
    }
}
